import javax.swing.*;
import java.lang.*;
import java.awt.*;

/**
  * DrawThread is a private class used by DataView to draw glyphs in a thread asyncronously.
  */

class DrawThread extends Thread
{
	DataView _dv;
	boolean continuerun;

	/** DrawThread is constructed with the DataView-object in use. */

	DrawThread(DataView dv)
	{
		_dv = dv;
		continuerun = true;
	}

	/** run method draws all the glyphs in the DataView by calling drawFirst, drawNext and drawFinal. */
	public void run() 
	{
		if(!_dv.drawFirst()) return;
		try {
		  int i = 0;
		  while(_dv.drawNext() && continuerun) 
		    if(i++ % 5 == 0) 
		      sleep(1); // Must leave some time for UI thread. yield() did not perform well enough.
		} catch(InterruptedException e) {
		}
		if(continuerun) _dv.drawFinal();
	}
	
	public void stopthread()
	{
		continuerun = false;
	}
}

/**
  * DataView is a base class for all the dataviews in the program.
  * It implements a mechanism for drawing glyphs asynchronously in a working thread.
  * Derived classes implement three methods: drawFirst, drawNext and drawFinal.
  */

public abstract class DataView extends JPanel
{
	DrawThread thread;

	public DataView()
	{
		thread = null;
	}

	/** startDraw() stops the drawing thread if it's active and starts a new one. */
          
	public boolean startDraw()
	{
		if(thread != null) {
			thread.stopthread();
			try {
				thread.join();
			} catch(InterruptedException e) {
			}
		}
		
		thread = new DrawThread(this);
		try {
			thread.start();
		} catch (IllegalThreadStateException e) {
			return false;
		}
		return true;
	}

	/** stopDraw() stops the drawing thread if it is active. */
	public boolean stopDraw()
	{
		if(thread != null) {
			thread.stopthread();
			try {
				thread.join();
			} catch(InterruptedException e) {
			}
		}
		thread = null;
		return true;
	}

	/** drawFirst is an abstract method which is called when the first glyph should be drawn. */	
	abstract public boolean drawFirst();

	/** drawNext is called to draw next glyph until the drawing is complete. */
	abstract public boolean drawNext();
	
	/** drawFinal is called after the final call to drawNext so that the DataView can make
          * final adjustments to the view. */
  abstract public boolean drawFinal();
}
