import java.awt.*;
import javax.swing.*;
import javax.swing.event.*;
import java.awt.event.*;
import java.lang.Math.*;

/**
  * Ruler component draws a horizontal or vertical ruler.
  * The component is given the minimum and maximum data values
  * and it calculates the correct number of ticks using the
  * component width and height.
  * 
  * @author Jaakko Haakana, Jari-Ville Lperi
  *
  */

class Ruler extends JComponent
{
  double min, max;
  boolean hor; // Horizontal (or vertical)
  double m_tickdelta;
  double m_tickstart;
  int m_tickcount;

  public double getTickDelta() { return m_tickdelta; }
  public double getTickStart() { return m_tickstart; }
  public int getTickCount() { return m_tickcount; }

  /** Label length in pixels. Calculated by calculateLabels(). */
  int m_labellen;

  /** Font used to draw the ruler tick strings. */
  Font font;

  /** Component constructor.
    * @param h True if component should be horizontal, false if vertical.
    */
  Ruler(boolean h, double mi, double ma)
    {
      font = new Font("Monospaced", Font.PLAIN, 8);
      if(font == null) {
	System.out.println("NO FONT!");
      }
      setMin(mi);
      setMax(ma);
      min = 0;
      max = 100.0;
      hor = h;
      calculateSize();
    }
  
  public void calculateAll()
    {
      calculateSize();
      calculateLabels();
      calculateTicks();
    }
  
  /** 
    * Function is used to (re-)calculate the correct size of the component.
    * based on the maximum and minimum values.
    */
  void calculateSize()
    {
      int fontheight = 16;
      int fontwidth = 10;
      Graphics g = getGraphics();
      if(g != null) 
      {
	g.setFont(font);
	FontMetrics fm = g.getFontMetrics();
	if(fm != null) {
	  fontheight = fm.getHeight();
	  fontwidth = fm.stringWidth("0123456789")/10;
	}
      }

      if(hor) {
	int hei = fontheight * 2 + 5;
	setMinimumSize(new Dimension(100, hei));
	setPreferredSize(new Dimension(0, hei));
	setMaximumSize(new Dimension(0, hei));
      } else {
	String s1 = (new Double(min)).toString();
	String s2 = (new Double(max)).toString();
	int wid = Math.max(fontwidth * s1.length(), fontwidth * s2.length()) + 15;
	setMinimumSize(new Dimension(wid, 100));
	setPreferredSize(new Dimension(wid, 0));
	setMaximumSize(new Dimension(wid, 0));
      }
    }
  
  
  /**
    * This function determines how much space should be allocated 
    * for one label.
    * 
    */
  
  void calculateLabels()
    {
      String s1 = (new Double(min)).toString();
      String s2 = (new Double(max)).toString();
      FontMetrics fm = getGraphics().getFontMetrics();
      if(hor)
	m_labellen = Math.max(fm.stringWidth(s1), fm.stringWidth(s2)) + 20;
      else
	m_labellen = fm.getHeight() + 25;
    }
  
  /** paint() paints the component with the current settings. */
  
  public void paint(Graphics g)
    {
      calculateAll();
      
      g.clearRect(0, 0, getWidth(), getHeight());
      g.setColor(new Color(0, 0, 0));
      if(hor) {
	g.drawLine(0, 0, getWidth(), 0); // draws the baseline
	for(int i=0;i<m_tickcount;i++)
	  {
	    int x = (int)((m_tickstart-min)/(max-min)*(double)getWidth()) + 
	      i*(int)(m_tickdelta/(max-min)*(double)getWidth());

	    g.drawLine(x, getHeight()-g.getFontMetrics().getHeight()-4, x, 0);
	    String s = (new Double(m_tickstart + (m_tickdelta * (double)i))).toString();
	    int l = g.getFontMetrics().stringWidth(s);
	    int dx = x - l / 2;
	    if(i==0) {
	      if(dx<0) {
		dx = 0;
	      }
	    } else if(i==m_tickcount-1) {
	      if(dx+l>getWidth()) {
		dx = getWidth() - l - 2;
	      }
	    }

	    g.drawString(s, dx, getHeight()-2);
	  }
	//		g.drawString((new Double(min)).toString(), 0, getHeight()-2);
	//		g.drawString((new Double(max)).toString(), getWidth()-50, getHeight()-2);
      } else {
	for(int i=0;i<m_tickcount;i++)
	  {
	    int y = getHeight() - ((int)((m_tickstart-min)/(max-min)*(double)getHeight()) + 
				   i*(int)(m_tickdelta/(max-min)*(double)getHeight()));
	    g.drawLine(getWidth()-10, y, getWidth(), y);
	    String s = (new Double(m_tickstart + (m_tickdelta * (double)i))).toString();
	    g.drawString(s, 2, y + g.getFontMetrics().getHeight()/2);
	  }
	g.drawLine(getWidth()-1, 0, getWidth()-1, getHeight());
      }
    }
  
  /** setMin() sets the minimum data value and repaints the component. */
  
  public void setMin(double m)
    {
      min = m;
      calculateSize();
      repaint();
    }
  
  /** setMax() sets the maximum data value and repaints the component. */
  
  public void setMax(double m)
    {
      max = m;
      calculateSize();
      repaint();
    }
  
  /** estimate() is a helper function used to calculate the correct number
    * of ticks the component should draw.
    */
  
  double estimate(double a,double ra)
    {
      int x = 0;
      if(ra > 1) 
	{
	  double temp=10;
	  for(x=1;temp>1;x++)
	    temp=Math.abs(a)/Math.pow(10,x);
	  x--;
	  
	  double prev;
	  double iso = Math.pow(10,x);
	  prev = iso;
	  iso = Math.pow(10,x) / 2;
	  if ((iso-a < 0) || ((ra / iso) > ra/a)) return prev;
	  prev = iso;
	  iso = Math.pow(10,x) / 5;
	  if ((iso-a < 0) || ((ra / iso) > ra/a)) return prev;
	  prev = iso;
	  iso = Math.pow(10,x) / 10;
	  if ((iso-a < 0) || ((ra / iso) > ra/a)) return prev;
	  prev = iso;
	  iso = Math.pow(10,x) / 20;
	  if ((iso-a < 0) || ((ra / iso) > ra/a)) return prev;
	  prev = iso;
	  iso = Math.pow(10,x) / 50;
	  if ((iso-a < 0) || ((ra / iso) > ra/a)) return prev;
	  return iso;
	  
	  
	} else {
	  double temp = 0;
	  for(x=-1;temp<1;x--)
	    temp = Math.abs(a)/Math.pow(10,x);
	  x++;
	  
	  double prev;
	  double iso = Math.pow(10,x);
	  prev=iso;
	  iso = Math.pow(10,x) / 2;
	  if ((iso-a < 0) || ((ra / iso) > ra/a)) return prev;
	  prev = iso;
	  iso = Math.pow(10,x) / 5;
	  if ((iso-a < 0) || ((ra / iso) > ra/a)) return prev;
	  prev = iso;
	  iso = Math.pow(10,x) / 10;
	  if ((iso-a < 0) || ((ra / iso) > ra/a)) return prev;
	  prev = iso;
	  iso = Math.pow(10,x) / 20;
	  if ((iso-a < 0) || ((ra / iso) > ra/a)) return prev;
	  prev = iso;
	  iso = Math.pow(10,x) / 50;
	  if ((iso-a < 0) || ((ra / iso) > ra/a)) return prev;
	  return iso;
	}
    }
  
  /** 
    * calculateTicks() performs the calculations for correct number of ticks.
    */

  void calculateTicks()
    {
      int a = hor ? getWidth() : getHeight();
      int maxticks = a / m_labellen;
      
      if(maxticks == 0) return; // TODO: Fix this
      
      double range = max - min;
      double minticklen = range / (double)maxticks;
      
      double tick = estimate(minticklen,range);
      
      double j = Math.abs(min) % tick;
      if(j > 0) 
	{
	  j = Math.floor((min + tick) / tick) * tick;
	} else {
	  j = min;
	}
      
      m_tickstart = j;
      m_tickdelta = tick;
      m_tickcount = (int) ((max - m_tickstart) / (double)tick) + 1;
    }
}
