import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;

/**
  * GlyphCPSelector implements ActionListener interface for the components where
  * user selects which data attributes are bound to which glyph properties.
  * When user changes the attribute for a glyph property actionPerformed() gets
  * called and the object sets the Glyph's property's data values (datamin, datamax).
  * The dataview is also updated.
  */

class GlyphCPSelector implements ActionListener
{
  Glyph2D glyph;
  JComboBox cb;
  int attrnum;
  TabDataSet dataset;
  DataView dataview;
  boolean isFrozen;

  GlyphCPSelector(int a, Glyph2D g, JComboBox b, TabDataSet ds, DataView dv)
    {
      attrnum = a;
      glyph = g;
      cb = b;
      dataset = ds;
      dataview = dv;
      isFrozen=false;
    }

  public void freeze() {
  	isFrozen = true;
  }
  public void melt() {
  	isFrozen = false;
  }
  public boolean getFrozen() {
  	return isFrozen;
  }

  /** Called when user changes the attribute selection for particular glyph property.
    * The property is set and the DataView is redrawn. */
  public void actionPerformed(ActionEvent e)
    {
      glyph.setPropertyDataAttribute(attrnum, cb.getSelectedIndex());
      glyph.setPropertyMin(attrnum, dataset.getMinDouble(cb.getSelectedIndex()));
      glyph.setPropertyMax(attrnum, dataset.getMaxDouble(cb.getSelectedIndex()));
      if (!isFrozen && e != null)
	dataview.startDraw(); // Update the data view.
    }

  public void update()
    {
      actionPerformed(null);
    }
}

/** Implements a control panel for TabDataView2D.
  * @see TabDataView2D
  */

public class TabDataView2DControlPanel extends DataViewControlPanel 
  implements ActionListener, GlyphControlActionListener, Constants
{
  TabDataSet ds;
  Glyph2D glyph; // Selected glyph.
  Glyph2DFactory gfac;
  JComboBox xaxis, yaxis; // Components for axis selection
  JComboBox cmbConstr1, cmbConstr2;
  JTextField txtConstr1min, txtConstr1max, txtConstr2min, txtConstr2max;   
  TabDataView2D dataview;

	/** TabDataView2DControlPanel is constructed with TabDataSet and TabDataView2D objects. 
	  * The constructor lays out the control panel GUI. It asks the available glyphs
	  * from Glyph2DFactory. */

	TabDataView2DControlPanel(TabDataSet dataset, TabDataView2D dv)
	{
		glyph = new BallGlyph2D(); // default glyph
		gfac = new Glyph2DFactory();

		dataview = dv;
		ds = dataset;

		dataview.setXAxis(0);
		dataview.setYAxis(1);
//		setSize(new Dimension(100, 100));

		JPanel glyphsel = new JPanel();
		glyphsel.setLayout(new BoxLayout(glyphsel, BoxLayout.X_AXIS));
		glyphsel.add(new JLabel("Glyph"));

		setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
		
		glyphsel.add(new JComboBox(gfac.getGlyphNames()));
		Vector attributes = new Vector(dataset.getNumberofAttributes());
		for(int i=0;i<dataset.getNumberofAttributes();i++) 
		{
		  attributes.add(dataset.getAttributeName(i));
		}
		add(glyphsel);

		JPanel xaxissel = new JPanel();
		xaxissel.setLayout(new BoxLayout(xaxissel, BoxLayout.X_AXIS));

		xaxissel.add(new JLabel("X-Axis"));
		xaxis = new JComboBox(attributes);
		xaxis.addActionListener(this);
		xaxis.setActionCommand("XAXIS");
		xaxissel.add(xaxis);
		add(xaxissel);

		JPanel yaxissel = new JPanel();
		yaxissel.setLayout(new BoxLayout(yaxissel, BoxLayout.X_AXIS));

		yaxissel.add(new JLabel("Y-axis"));
		yaxis = new JComboBox(attributes);
		yaxis.addActionListener(this);
		yaxis.setActionCommand("YAXIS");
		yaxissel.add(yaxis);
		add(yaxissel);

		yaxis.setSelectedIndex(1);

		add(new JLabel("Glyph properties"));
		int gprops = glyph.getNumberofProperties();
		for(int i=0;i<gprops;i++)
		{
		  JPanel proppanel = new JPanel();
		  proppanel.setLayout(new BoxLayout(proppanel, BoxLayout.X_AXIS));
		  proppanel.add(new JLabel(glyph.getPropertyName(i)));
		  JComboBox cb = new JComboBox(attributes);
		  GlyphCPSelector sel = new GlyphCPSelector(i, glyph, cb, dataset, dataview);
		  sel.update();
		  cb.addActionListener(sel);
		  proppanel.add(cb);
		  Component c = glyph.getPropertyComponent(i, this);
		  if(c != null) {
		    proppanel.add(c);
		  }
		  add(proppanel);
		}

                // constraint panel
                Vector attributesNotClass = new Vector();
                attributesNotClass.add("no attributes selected");
                for(int i=0;i<dataset.getNumberofAttributes();i++)
                {
                  if (dataset.getAttributeType(i) != 2) //not a class
                     attributesNotClass.add(dataset.getAttributeName(i));
                }      
                JPanel pnlConstr = new JPanel();
                pnlConstr.setLayout(new GridLayout(2,1));
 
                JPanel pnlConstr1 = new JPanel();
                pnlConstr1.setLayout(new BoxLayout(pnlConstr1, BoxLayout.X_AXIS));
                pnlConstr1.add(new JLabel("Constraint 1 "));
                cmbConstr1 = new JComboBox(attributesNotClass);
                cmbConstr1.addActionListener(this);
                cmbConstr1.setActionCommand("CONSTR1ATTRCHG");
                pnlConstr1.add(cmbConstr1);
                txtConstr1min = new JTextField();  
                txtConstr1min.setActionCommand("CONSTR1VAL");
                txtConstr1min.addActionListener(this);
                pnlConstr1.add(txtConstr1min);
                pnlConstr1.add(new JLabel("-"));
                txtConstr1max = new JTextField();
                txtConstr1max.setActionCommand("CONSTR1VAL");
                txtConstr1max.addActionListener(this);
                pnlConstr1.add(txtConstr1max);

                pnlConstr.add(pnlConstr1);              

                JPanel pnlConstr2 = new JPanel();
                pnlConstr2.setLayout(new BoxLayout(pnlConstr2, BoxLayout.X_AXIS));
                pnlConstr2.add(new JLabel("Constraint 2 "));
                cmbConstr2 = new JComboBox(attributesNotClass);
                cmbConstr2.addActionListener(this);
                cmbConstr2.setActionCommand("CONSTR2ATTRCHG");
                pnlConstr2.add(cmbConstr2);
                txtConstr2min = new JTextField();
                txtConstr2min.setActionCommand("CONSTR2VAL");
                txtConstr2min.addActionListener(this);
                pnlConstr2.add(txtConstr2min);
                pnlConstr2.add(new JLabel("-"));
                txtConstr2max = new JTextField();
                txtConstr2max.setActionCommand("CONSTR2VAL");
                txtConstr2max.addActionListener(this);
                pnlConstr2.add(txtConstr2max);
  
                pnlConstr.add(pnlConstr2);     

                add(pnlConstr); 
                //end of constraint panel

		glyph.setPropertyDataAttribute(0, 0);
		glyph.setPropertyDataAttribute(1, 0);
		glyph.setPropertyMin(0, dataset.getMinDouble(0));
		glyph.setPropertyMax(0, dataset.getMaxDouble(0));
		glyph.setPropertyMin(1, dataset.getMinDouble(0));
		glyph.setPropertyMax(1, dataset.getMaxDouble(0));

		dataview.setGlyph(glyph);
	}
  
  public void actionPerformed(ActionEvent e)
    {
      if(e.getActionCommand() == "XAXIS") {
	dataview.setXAxis(xaxis.getSelectedIndex());
      }
      else
      if(e.getActionCommand() == "YAXIS") {
	dataview.setYAxis(yaxis.getSelectedIndex());
      }
      else
      if(e.getActionCommand() == "CONSTR1ATTRCHG") {
        this.constraintAttrChg(1);
      }            
      else
      if(e.getActionCommand() == "CONSTR2ATTRCHG") {
        this.constraintAttrChg(2);
      }      
      else
      if(e.getActionCommand() == "CONSTR1VALUE") {
        this.constraintValueChg(1);
      }
      else
      if(e.getActionCommand() == "CONSTR2VALUE") {
        this.constraintValueChg(2);
      }
      else
        if (DEBUG) System.out.println("Unknown action: "+e.getActionCommand());
   }

   public void constraintValueChg(int constrNo) {
      JComboBox cmbTmp;
      JTextField txtMin, txtMax;
      TabCursor tmpCursor = (TabCursor) dataview.getTabCursor();
      int attrNo = -1;
      if (constrNo == 1) {
         cmbTmp = cmbConstr1;
         txtMin = txtConstr1min;
         txtMax = txtConstr1max;
       }
      else {
         cmbTmp =  cmbConstr2;
         txtMin = txtConstr2min;
         txtMax = txtConstr2max;
     }
     tmpCursor.clearConstraint(constrNo);
     //searching for the number of the attribute
     int i = 0;
     String selName = (String) cmbTmp.getItemAt(cmbTmp.getSelectedIndex());

     while(i < ds.getNumberofAttributes()) {
        if (ds.getAttributeName(i).equals(selName))
          attrNo = i;
        ++i;
      }       
   }

   public void constraintAttrChg(int constrNo) {
      JComboBox cmbTmp;
      JTextField txtMin, txtMax;
      TabCursor tmpCursor = (TabCursor) dataview.getTabCursor();
      int attrNo = -1;
      if (constrNo == 1) {
         cmbTmp = cmbConstr1;
         txtMin = txtConstr1min; 
         txtMax = txtConstr1max;
       }
      else {
         cmbTmp =  cmbConstr2;
         txtMin = txtConstr2min; 
         txtMax = txtConstr2max;
     }
     tmpCursor.clearConstraint(constrNo);     
     //searching for the number of the attribute
     int i = 0;
     String selName = (String) cmbTmp.getItemAt(cmbTmp.getSelectedIndex());

     while(i < ds.getNumberofAttributes()) { 
        if (ds.getAttributeName(i).equals(selName)) 
          attrNo = i;
        ++i;
      }      
     if (attrNo == -1) {//no constraints selected
       txtMin.setText(null);
       txtMax.setText(null);
     }
     else  {
       if (ds.getAttributeType(attrNo) == 0) { //long
          txtMin.setText(String.valueOf(ds.getMinLong(attrNo)));
          txtMax.setText(String.valueOf(ds.getMaxLong(attrNo)));
        }
       else
       if (ds.getAttributeType(attrNo) == 1) { //double
          txtMin.setText(String.valueOf(ds.getMinDouble(attrNo)));
          txtMax.setText(String.valueOf(ds.getMaxDouble(attrNo)));
       }
     }
  } 

  public void refreshDataView()
    {
      // Someone tells us that our dataview should be redrawn. Let's do it.
      dataview.startDraw();
    }
}
